import time
from multiprocessing import Pool
from typing import List, Optional

import typer

from gdshoplib.apps.product import Product
from gdshoplib.packages.cache import KeyDBCache
from gdshoplib.services.notion.database import Database
from gdshoplib.services.notion.notion import Notion

app = typer.Typer()


@app.command()
def clean(id: Optional[str] = typer.Option(None)):
    # TODO: сделать удаление по ID
    KeyDBCache().clean(r"[blocks|pages|databases]*")


@app.command()
def listen(topic: List[str]):
    for message in KeyDBCache().subscribe(topic):
        print(message)


@app.command()
def warm(
    only_exists: bool = typer.Option(False),
    single: bool = typer.Option(False),
    only_edited: bool = typer.Option(True),
    sku: Optional[str] = typer.Option(None),
    loop_iteration: Optional[int] = typer.Option(None),
):
    while True:
        if sku:
            cache_warm_func(Product.get(sku).id)
            return

        if single:
            with Database(
                Product.SETTINGS.PRODUCT_DB, notion=Notion(caching=True)
            ) as database:
                params = {}
                if only_edited and database.get_update_time():
                    print(f"Фильтрация от даты: {database.get_update_time()}")
                    params = database.edited_filter()

                for product in database.pages(params=params):
                    skipped = False
                    if only_exists:
                        if KeyDBCache().exists(product["id"]):
                            print(f"{product['id']}: SKIPPED")
                            skipped = True

                    if not skipped:
                        cache_warm_func(product["id"])
        else:
            with Pool(3) as p:
                with Database(
                    Product.SETTINGS.PRODUCT_DB, notion=Notion(caching=True)
                ) as database:
                    params = {}
                    if only_edited and database.get_update_time():
                        print(f"Фильтрация от даты: {database.get_update_time()}")
                        params = database.edited_filter()

                    for product in database.pages(params=params):
                        skipped = False
                        if only_exists:
                            if KeyDBCache().exists(product["id"]):
                                print(f"{product['id']}: SKIPPED")
                                skipped = True

                        if not skipped:
                            p.apply_async(cache_warm_func, (product["id"],))
                p.close()
                p.join()

        if loop_iteration:
            print("-" * 20)
            time.sleep(loop_iteration)
        else:
            break


@app.command()
def count():
    print(KeyDBCache().count())


@app.command()
def check(single: bool = typer.Option(False)):
    if single:
        for product in Database(Product.SETTINGS.PRODUCT_DB).pages():
            cache_check_action(product["id"])
    else:
        with Pool(3) as p:
            for product in Database(Product.SETTINGS.PRODUCT_DB).pages():
                p.apply_async(cache_check_action, (product["id"],))
            p.close()
            p.join()


#####


def cache_check_action(id):
    for block in Notion().get_blocks(id):
        exists = KeyDBCache().exists(block["id"])
        print(f"{block['id']}: {exists}")


def cache_warm_func(id):
    product = Product(id)
    product.warm()
    print(f"{product.sku}: {product.last_edited_time}")



import datetime
from typing import List

import backoff
import requests as r
from loguru import logger
from pydantic import BaseModel

from gdshoplib.core.settings import AVITOSettings


class Avito:
    BASE_URL = "https://api.avito.ru/"

    def __init__(self) -> None:
        self._token = None
        self.settings = AVITOSettings()

    def get_access_token(self):
        response = r.post(
            f"{self.BASE_URL}token",
            params={
                "client_id": self.settings.AVITO_CLIENT_ID,
                "client_secret": self.settings.AVITO_SECRET,
                "grant_type": "client_credentials",
            },
        )
        if not response.ok:
            logger.warning(r.json())
            assert (
                False
            ), f"Запрос авторизации прошел с ошибкой {response.status_code}/n"
        return response.json()["access_token"]

    def get_headers(self):
        if not self._token:
            self._token = self.get_access_token()

        return {"Authorization": f"Bearer {self._token}"}

    def get_avito_ids(self, products):
        response = self.make_request(
            "autoload/v2/items/avito_ids",
            method="GET",
            params={"query": ",".join((product.sku for product in products))},
        )
        return response["items"]

    def get_statistic(self, date_from=None, date_to=None) -> str:
        response = self.make_request(
            f"stats/v1/accounts/{self.settings.AVITO_USER_ID}/items",
            method="POST",
            params={
                "dateFrom": (
                    date_from or datetime.datetime.now() - datetime.timedelta(days=1)
                ).strftime("%Y-%m-%d"),
                "dateTo": (date_to or datetime.datetime.now()).strftime("%Y-%m-%d"),
                "fields": ["uniqViews", "uniqContacts", "uniqFavorites"],
                "periodGrouping": "day",
            },
        )
        result = []
        for item in response["result"].get("items", []):
            result.append(
                AvitoStatsItem(
                    avito_id=item["itemId"],
                    points=[
                        AvitoStatsPoint(
                            date=point["date"],
                            contacts=point["uniqContacts"],
                            favorites=point["uniqContacts"],
                            views=point["uniqContacts"],
                        )
                        for point in item["stats"]
                    ],
                )
            )

        return result

    @backoff.on_exception(
        backoff.expo,
        (
            r.ReadTimeout,
            r.ConnectTimeout,
            r.Timeout,
            r.JSONDecodeError,
            r.ConnectionError,
        ),
        max_tries=8,
    )
    def make_request(self, path, *, method, params=None):
        _path = f"{self.BASE_URL}{path}"
        _params = (
            dict(params=params) or {}
            if method.upper() == "GET"
            else dict(json=params) or {}
        )

        _r = r.request(
            method,
            _path,
            headers=self.get_headers(),
            timeout=10.0,
            **_params,
        )
        if not _r.ok:
            if r.status_codes == 401:
                self._token = None
            logger.warning(_r.json())
            assert (
                False
            ), f"Запрос {method.upper()} {_path} прошел с ошибкой {_r.status_code}/n"
        return _r.json()


class AvitoStatsPoint(BaseModel):
    date: datetime.date
    contacts: int
    favorites: int
    views: int


class AvitoStatsItem(BaseModel):
    avito_id: int
    points: List[AvitoStatsPoint]



import typer

from gdshoplib.cli.product.cache import app as cache
from gdshoplib.cli.product.controle import app as controle
from gdshoplib.cli.product.description import app as description
from gdshoplib.cli.product.feed import app as feed
from gdshoplib.cli.product.media import app as media
from gdshoplib.cli.product.price import app as price

app = typer.Typer()

app.add_typer(cache, name="cache")
app.add_typer(controle, name="controle")
app.add_typer(description, name="description")
app.add_typer(feed, name="feed")
app.add_typer(media, name="media")
app.add_typer(price, name="price")

if __name__ == "__main__":
    app()



import typer

from gdshoplib.cli.crm.application import app as crm
from gdshoplib.cli.finance.application import app as finance
from gdshoplib.cli.product.application import app as product
from gdshoplib.cli.service.application import app as service

app = typer.Typer()

app.add_typer(crm, name="crm")
app.add_typer(finance, name="finance")
app.add_typer(product, name="product")
app.add_typer(service, name="service")

if __name__ == "__main__":
    app()



import logging

import backoff
import requests as r

logger = logging.getLogger(__name__)


class RequestManager:
    BASE_URL = "https://api.notion.com/v1/"

    @backoff.on_exception(
        backoff.expo,
        (
            r.ReadTimeout,
            r.ConnectTimeout,
            r.Timeout,
            r.JSONDecodeError,
            r.ConnectionError,
        ),
        max_tries=8,
    )
    def make_request(self, path, *, method, params=None):
        _path = f"{self.BASE_URL}{path}"
        _params = (
            dict(params=params) or {}
            if method.upper() == "GET"
            else dict(json=params) or {}
        )

        _r = r.request(
            method,
            _path,
            headers=self.get_headers(),
            timeout=10.0,
            **_params,
        )
        if not _r.ok:
            logger.warning(_r.json())
            if _r.status_code == 404:
                raise NotFoundException
            assert (
                False
            ), f"Запрос {method.upper()} {_path} прошел с ошибкой {_r.status_code}/n"
        return _r.json()

    def pagination(self, url, *, params=None, **kwargs):
        _params = params or {}
        response = None
        result = []
        while True:
            response = self.make_request(url, params=_params, **kwargs)
            result.extend(response["results"])

            if not response.get("has_more"):
                return result
            _params["start_cursor"] = response["next_cursor"]


class NotFoundException(Exception):
    ...



# Делаем транслитерацию
symbols = (
    "абвгдеёжзийклмнопрстуфхцчшщъыьэюяАБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ ",
    (
        *list("abvgdee"),
        "zh",
        *list("zijklmnoprstuf"),
        "kh",
        "z",
        "ch",
        "sh",
        "sh",
        "",
        "y",
        "",
        "e",
        "yu",
        "ya",
        *list("ABVGDEE"),
        "ZH",
        *list("ZIJKLMNOPRSTUF"),
        "KH",
        "Z",
        "CH",
        "SH",
        "SH",
        *list("_Y_E"),
        "YU",
        "YA",
        " ",
    ),
)

coding_dict = {source: dest for source, dest in zip(*symbols)}


def transliterate(x):
    return "".join([coding_dict[i] for i in x])



import logging
import webbrowser

import requests

from gdshoplib.core.settings import VKSettings

logger = logging.getLogger(__name__)


class VKNotFoundException(Exception):
    ...


class VK:
    def __init__(self):
        self.settings = VKSettings()

    def auth_headers(self):
        return {"Authorization": f"Bearer {self.settings.VK_USER_ACCESS_TOKEN}"}

    def request(self, method, params=None, url=None, http_method=None, files=None):
        _params = {"v": self.settings.VK_API_VERSION}
        _params.update(params or {})

        r = requests.request(
            http_method or "get",
            url or f"{self.settings.VK_BASEPATH}{method}",
            params=_params,
            headers=self.auth_headers(),
            files=files or {},
        )

        if not r.ok:
            logger.warning(r.json())
            if r.status_code == 404:
                raise VKNotFoundException

            assert False, f"Запрос {method} {params} прошел с ошибкой {r.status_code}/n"

        return r.json()

    def get_oauth_code(self):
        # Получить токен для работы
        # Принять код, получить access_token и напечатать его
        webbrowser.open(self.get_user_code_url())

    def get_access_token(self, code):
        r = requests.get(self.get_user_access_token_url(code))
        assert r.ok, "Запрос прошел с ошибкой"

        print(r.json())

    def get_user_code_url(self):
        return self.create_url(
            "https://oauth.vk.com/authorize",
            dict(
                client_id=self.settings.VK_CLIENT_ID,
                display="page",
                scope=self.settings.VK_USER_SCOPE,
                response_type="code",
                v=self.settings.VK_API_VERSION,
            ),
        )

    def get_user_access_token_url(self, code):
        assert self.settings.VK_SECRET_KEY, "VK_SECRET_KEY не определен"
        return self.create_url(
            "https://oauth.vk.com/access_token",
            dict(
                client_id=self.settings.VK_CLIENT_ID,
                client_secret=self.settings.VK_SECRET_KEY,
                code=code,
            ),
        )

    @staticmethod
    def create_url(base_path, params):
        path = f"{base_path}?"
        for k, v in params.items():
            path += f"{k}={v}&"

        return path[:-1]



import typer

from gdshoplib.services.vk.market import VKMarket
from gdshoplib.services.vk.vk import VK

app = typer.Typer()


@app.command()
def access_code(code=None):
    if not code:
        VK().get_oauth_code()
        code = typer.prompt("Код")

    print(VK().get_access_token(code))


@app.command()
def health():
    assert VKMarket().list(), "Запрос в VK не выполняется"
    print("OK")



import typer

from gdshoplib.apps.finance.storage import Storage
from gdshoplib.apps.product import Product

app = typer.Typer()


@app.command()
def amount(base_price="now"):
    print(f"{base_price}: {Storage().amount(Product.query(), base_price=base_price)}")



import typer

from gdshoplib.apps.product import Product

app = typer.Typer()


@app.command()
def sku_set():
    for page in Product.query(
        params={
            "filter": {
                "and": [
                    {"property": "Наш SKU", "rich_text": {"is_empty": True}},
                    {"property": "Цена (eur)", "number": {"is_not_empty": True}},
                ]
            }
        },
    ):
        sku = page.generate_sku()
        while not Product.query(filter={"sku": sku}):
            sku = page.generate_sku()

        page.notion.update_prop(
            page.id, params={"properties": {"Наш SKU": [{"text": {"content": sku}}]}}
        )
        print(Product(page.id).sku)



import re
from random import randrange

from gdshoplib.apps.products.description import ProductDescription
from gdshoplib.apps.products.media import ProductMedia
from gdshoplib.apps.products.price import Price
from gdshoplib.core.settings import ProductSettings
from gdshoplib.services.notion.database import Database
from gdshoplib.services.notion.page import Page


class Product(Page):
    SETTINGS = ProductSettings()

    def __init__(self, *args, **kwargs):
        self._images = None
        self._videos = None
        self._price = None
        self._description = None
        self._kit = None
        self._badges = None

        super(Product, self).__init__(*args, **kwargs)

    @classmethod
    def get(cls, sku):
        page = [
            page
            for page in Database(cls.SETTINGS.PRODUCT_DB).pages(
                params={
                    "filter": {
                        "property": "Наш SKU",
                        "rich_text": {
                            "equals": sku,
                        },
                    }
                }
            )
        ]
        if not page:
            return
        if len(page) > 1:
            raise SKUDuplicate
        page = page[0]

        return cls(page["id"], notion=page.notion, parent=page.parent)

    @classmethod
    def query(cls, filter=None, params=None, notion=None):
        for page in Database(cls.SETTINGS.PRODUCT_DB, notion=notion).pages(
            filter=filter, params=params
        ):
            yield cls(page["id"], notion=page.notion, parent=page.parent)

    @property
    def media(self):
        return [*self.images, *self.videos]

    @property
    def videos(self):
        if not self._videos:
            self._videos = list(
                ProductMedia(block.id, notion=self.notion, parent=self)
                for block in self.blocks(filter={"type": "video"})
            )
        return self._videos

    @property
    def images(self):
        if not self._images:
            self._images = list(
                ProductMedia(block.id, notion=self.notion, parent=self)
                for block in self.blocks(filter={"type": "image"})
            )
        return self._images

    @property
    def price(self):
        if not self._price:
            self._price = Price(self)
        return self._price

    @property
    def description(self):
        if not self._description:
            self._description = ProductDescription(self)
        return self._description

    @staticmethod
    def split(source):
        result = []
        if not source:
            return []

        for tag in source.split("/"):
            result.append(tag.strip())

        return result

    @property
    def tags(self):
        result = []
        for tag in self.split(self.tags_field):
            result.append(re.sub(r"[\W\s]", "", tag).lower())
        return result

    @property
    def specifications(self):
        result = []
        for spec in self.split(self.specifications_field):
            result.append(spec.capitalize())
        return result

    @property
    def notes(self):
        result = []
        for spec in self.split(self.notes_field):
            result.append(spec.capitalize())
        return result

    @property
    def kit(self):
        if not self.kit_field:
            return

        if not self._kit:
            self._kit = []
            for page in self.kit_field:
                if self.id != page.id:
                    self._kit.append(
                        Product(page.id, notion=self.notion, parent=self.parent)
                    )
        return self._kit

    @property
    def badges(self):
        if not self.badge_field:
            return

        if not self._badges:
            self._badges = []
            for page in self.badge_field:
                if self.id != page.id:
                    self._badges.append(
                        Page(page.id, notion=self.notion, parent=self.parent)
                    )
        return self._badges

    def available(self):
        return bool(self.quantity)

    def generate_sku(self):
        # Сгенерировать SKU на основе продукта
        # Категория.Бренд.Цена_покупки.месяц_добавления.год_добавления.случайные_4_числа

        sku = (
            f"{self.brand.title.upper() if self.brand else ''}"
            f"{int(self.price.eur)}"
            f"{randrange(1111, 9999)}"
        )

        return re.sub(r"\W", "", sku)

    def __str__(self) -> str:
        return f"{self.__class__}: {self.sku}"

    def __repr__(self) -> str:
        return f"{self.__class__}: {self.sku}"

    def warm(self):
        self.price.now
        self.kit
        self.notes
        self.specifications
        self.tags
        self.media
        self.description
        self.brand.title
        self.description.warm_description_blocks()


class SKUDuplicate(Exception):
    # Если найден товар с одинаковым SKU
    ...
